@@ -1,23 +1,24 @@
 #!/usr/bin/env python
-"""
-Converts a mitmproxy dump file to a swagger schema.
-"""
+# -*- coding: utf-8 -*-
+"""Converts a mitmproxy dump file to a swagger schema."""
+import argparse
+import json
 import os
+import re
 import sys
 import traceback
-from mitmproxy.exceptions import FlowReadException
-import json
-import argparse
+import urllib
+from typing import Any, Sequence
+
 import ruamel.yaml
-import re
-from . import swagger_util
-from .har_capture_reader import HarCaptureReader, har_archive_heuristic
-from .mitmproxy_capture_reader import (
+from mitmproxy.exceptions import FlowReadException
+
+from mitmproxy2swagger import console_util, swagger_util
+from mitmproxy2swagger.har_capture_reader import HarCaptureReader, har_archive_heuristic
+from mitmproxy2swagger.mitmproxy_capture_reader import (
     MitmproxyCaptureReader,
     mitmproxy_dump_file_huristic,
 )
-from . import console_util
-import urllib
 
 
 def path_to_regex(path):
@@ -62,7 +63,7 @@ def detect_input_format(file_path):
     return MitmproxyCaptureReader(file_path, progress_callback)
 
 
-def main():
+def main(override_args: Sequence[str] | None = None):
     parser = argparse.ArgumentParser(
         description="Converts a mitmproxy dump file or HAR to a swagger schema."
     )
@@ -85,17 +86,23 @@ def main():
         action="store_true",
         help="Include examples in the schema. This might expose sensitive information.",
     )
+    parser.add_argument(
+        "-hd",
+        "--headers",
+        action="store_true",
+        help="Include headers in the schema. This might expose sensitive information.",
+    )
     parser.add_argument(
         "-f",
         "--format",
         choices=["flow", "har"],
         help="Override the input file format auto-detection.",
     )
-    args = parser.parse_args()
+    args = parser.parse_args(override_args)
 
     yaml = ruamel.yaml.YAML()
 
-    capture_reader = None
+    capture_reader: MitmproxyCaptureReader | HarCaptureReader
     if args.format == "flow" or args.format == "mitmproxy":
         capture_reader = MitmproxyCaptureReader(args.input, progress_callback)
     elif args.format == "har":
@@ -107,11 +114,13 @@ def main():
 
     # try loading the existing swagger file
     try:
-        with open(args.output, "r") as f:
+        base_dir = os.getcwd()
+        relative_path = args.output
+        abs_path = os.path.join(base_dir, relative_path)
+        with open(abs_path, "r") as f:
             swagger = yaml.load(f)
     except FileNotFoundError:
         print("No existing swagger file found. Creating new one.")
-        pass
     if swagger is None:
         swagger = ruamel.yaml.comments.CommentedMap(
             {
@@ -156,14 +165,15 @@ def main():
     path_template_regexes = [re.compile(path_to_regex(path)) for path in path_templates]
 
     try:
-        for f in capture_reader.captured_requests():
+        for req in capture_reader.captured_requests():
             # strip the api prefix from the url
-            url = f.get_url()
-            if not url.startswith(args.api_prefix):
+            url = req.get_matching_url(args.api_prefix)
+
+            if url is None:
                 continue
-            method = f.get_method().lower()
+            method = req.get_method().lower()
             path = strip_query_string(url).removeprefix(args.api_prefix)
-            status = f.get_response_status_code()
+            status = req.get_response_status_code()
 
             # check if the path matches any of the path templates, and save the index
             path_template_index = None
@@ -192,20 +202,29 @@ def main():
             )
 
             params = swagger_util.url_to_params(url, path_template_to_set)
-
+            if args.headers:
+                headers_request = swagger_util.request_to_headers(
+                    req.get_request_headers()
+                )
+                if headers_request is not None and len(headers_request) > 0:
+                    set_key_if_not_exists(
+                        swagger["paths"][path_template_to_set][method],
+                        "parameters",
+                        headers_request,
+                    )
             if params is not None and len(params) > 0:
                 set_key_if_not_exists(
                     swagger["paths"][path_template_to_set][method], "parameters", params
                 )
 
             if method not in ["get", "head"]:
-                body = f.get_request_body()
+                body = req.get_request_body()
                 if body is not None:
                     body_val = None
                     content_type = None
                     # try to parse the body as json
                     try:
-                        body_val = json.loads(f.get_request_body())
+                        body_val = json.loads(req.get_request_body())
                         content_type = "application/json"
                     except UnicodeDecodeError:
                         pass
@@ -214,7 +233,7 @@ def main():
                     if content_type is None:
                         # try to parse the body as form data
                         try:
-                            body_val_bytes = dict(
+                            body_val_bytes: Any = dict(
                                 urllib.parse.parse_qsl(
                                     body, encoding="utf-8", keep_blank_values=True
                                 )
@@ -250,7 +269,7 @@ def main():
                         )
 
             # try parsing the response as json
-            response_body = f.get_response_body()
+            response_body = req.get_response_body()
             if response_body is not None:
                 try:
                     response_json = json.loads(response_body)
@@ -260,7 +279,7 @@ def main():
                     response_json = None
                 if response_json is not None:
                     resp_data_to_set = {
-                        "description": f.get_response_reason(),
+                        "description": req.get_response_reason(),
                         "content": {
                             "application/json": {
                                 "schema": swagger_util.value_to_schema(response_json)
@@ -271,11 +290,31 @@ def main():
                         resp_data_to_set["content"]["application/json"][
                             "example"
                         ] = swagger_util.limit_example_size(response_json)
+                    if args.headers:
+                        resp_data_to_set["headers"] = swagger_util.response_to_headers(
+                            req.get_response_headers()
+                        )
+
                     set_key_if_not_exists(
                         swagger["paths"][path_template_to_set][method]["responses"],
                         str(status),
                         resp_data_to_set,
                     )
+            if (
+                "responses" in swagger["paths"][path_template_to_set][method]
+                and len(swagger["paths"][path_template_to_set][method]["responses"])
+                == 0
+            ):
+                # add a default response if there were no responses detected,
+                # this is for compliance with the OpenAPI spec
+                content_type = (
+                    req.get_response_headers().get("content-type") or "text/plain"
+                )
+
+                swagger["paths"][path_template_to_set][method]["responses"]["200"] = {
+                    "description": "OK",
+                    "content": {content_type: {}},
+                }
 
     except FlowReadException as e:
         print(f"Flow file corrupted: {e}")
