@@ -1,4 +1,6 @@
+# -*- coding: utf-8 -*-
 import urllib
+from typing import Any, List
 
 VERBS = [
     "add",
@@ -78,6 +80,35 @@ def url_to_params(url, path_template):
     return params
 
 
+def request_to_headers(headers: dict[str, List[Any]], add_example: bool = False):
+    """When given an url and its path template, generates the parameters
+    section of the request."""
+    params = []
+    if headers:
+        for key in headers:
+            h = {
+                "name": key,
+                "in": "header",
+                "required": False,
+                "schema": {"type": "number" if headers[key][0].isdigit() else "string"},
+            }
+            if add_example:
+                h["example"] = headers[key][0]
+            params.append(h)
+    return params
+
+
+def response_to_headers(headers):
+    header = {}
+    if headers:
+        for key in headers:
+            header[key] = {
+                "description": headers[key][0],
+                "schema": {"type": "number" if headers[key][0].isdigit() else "string"},
+            }
+    return header
+
+
 def value_to_schema(value):
     # check if value is a number
     if type(value) == int or type(value) == float:
